InvertibleComponentIdSet

Enum InvertibleComponentIdSet 

pub enum InvertibleComponentIdSet {
    Included(ComponentIdSet),
    Excluded(ComponentIdSet),
}
A set of bits that is either a finite set, or the set complement of a finite set.

Variants



Included(ComponentIdSet)

A finite InvertibleComponentIdSet that includes all components in the inner set.



Excluded(ComponentIdSet)

An unbounded InvertibleComponentIdSet that includes all components not in the inner set.

Implementations

Source

impl InvertibleComponentIdSet

pub const fn new() -> InvertibleComponentIdSet

Creates a new empty InvertibleSet.

pub const fn new_all() -> InvertibleComponentIdSet

Creates a new InvertibleSet that includes all components.

pub fn insert(&mut self, index: ComponentId)

Adds a ComponentId to the set.

pub fn remove(&mut self, index: ComponentId)

Removes a ComponentId from the set.

pub fn clear(&mut self)

Removes all ComponentIds from the set.

pub fn all(&mut self)

Adds all ComponentIds to the set.

pub fn contains(&self, index: ComponentId) -> bool

Returns true if the ComponentId is in the set.

pub fn is_clear(&self) -> bool

Returns true if the set is empty. Note that an unbounded set is never clear.

pub fn is_all(&self) -> bool

Returns true if the set contains all components. Note that a finite set is never fully set.

pub fn is_unbounded(&self) -> bool

Returns true if this is an unbounded set, or false if it is a finite set.

pub fn into_finite_set(self) -> Option<ComponentIdSet>

If this is a finite set, returns the set. If this is an unbounded set, returns None.

pub fn as_finite_set(&self) -> Option<&ComponentIdSet>

If this is a finite set, returns the set. If this is an unbounded set, returns None.

pub fn as_exclusion_set(&self) -> Option<&ComponentIdSet>

If this is an unbounded set, returns the set of components that are excluded from the set. If this is a finite set, returns None.

pub fn union_with(&mut self, other: &InvertibleComponentIdSet)

In-place union of two sets.

pub fn union( &self, other: &InvertibleComponentIdSet, ) -> InvertibleComponentIdSet

Returns the union of two sets.

pub fn difference_with(&mut self, other: &InvertibleComponentIdSet)

In-place difference of two sets.

pub fn difference( &self, other: &InvertibleComponentIdSet, ) -> InvertibleComponentIdSet

Returns the set difference of two sets.

pub fn intersect_with(&mut self, other: &InvertibleComponentIdSet)

In-place intersection of two sets.

pub fn intersection( &self, other: &InvertibleComponentIdSet, ) -> InvertibleComponentIdSet

Returns the intersection of two sets.

pub fn is_disjoint(&self, other: &InvertibleComponentIdSet) -> bool

Returns true if self has no elements in common with other. This is equivalent to checking for an empty intersection.

pub fn is_subset(&self, other: &InvertibleComponentIdSet) -> bool

Returns true if the set is a subset of another, i.e. other contains at least all the values in self.