has_conflicts

Function has_conflicts 

pub fn has_conflicts<Q>(components: &Components) -> Result<(), QueryAccessError>
where Q: QueryData,
Check Q for internal conflicts, panicking if there are any.

Returns an error if not all components are registered.