pub unsafe trait SingleEntityQueryData: IterQueryData { }Expand description
A QueryData that only accesses data from the current entity, the one passed to QueryData::fetch.
This is used as a bound in EntityRef::get_components and related APIs,
since they only have access to a single entity.
Safety
This QueryData must only access data from the current entity, and not any other entities.
Dyn Compatibility
This trait is not dyn compatible.
In older versions of Rust, dyn compatibility was called "object safety", so this trait is not object safe.